@@ -5,7 +5,7 @@
 ;; Author: Zajcev Evgeny <zevlg@yandex.ru>
 ;; Created: Mon Jan 12 13:14:32 MSK 2004
 ;; Keywords: xwem
-;; X-CVS: $Id: xwem-osd.el,v 1.3 2004/07/14 08:38:56 youngs Exp $
+;; X-CVS: $Id: xwem-osd.el,v 1.6 2005/01/01 04:42:58 youngs Exp $
 
 ;; This file is part of XWEM.
 
@@ -79,11 +79,26 @@
 ;;; Code:
 
 (eval-when-compile
-  (require 'cl)
-  (require 'xlib-xlib))
-(require 'xlib-xshape)
+  (require 'cl))
 
-(defconst xwem-osd-instance-types '(text line dots arc rect)
+(require 'xlib-xshape)
+(require 'xlib-tray)
+(require 'xlib-xpm)
+
+(require 'xwem-diagram)
+
+(defcustom xwem-osd-default-font "fixed"
+  "Default font for text drawed in osd.")
+
+(defcustom xwem-osd-default-color "black"
+  "Default color used to draw.")
+
+(defcustom xwem-osd-always-ontop t
+  "*Non-nil mean that OSD's winow will be always on top.")
+
+;;; Internal variables
+
+(defconst xwem-osd-instance-types '(text line dots arc rect icon)
   "List of valid types of osd instance.")
 
 
@@ -93,7 +108,24 @@
   (depth 0)				; depth
 
   xwin xmask
-  color)				; instance background color
+  color                                 ; instance background color
+
+  plist)                                ; User defined plist
+
+(defsubst xwem-osd-instance-put-prop (osin prop val)
+  "In OSD's instance OSIN properties list put property PROP with value VAL."
+  (setf (xwem-osd-instance-plist osin)
+        (plist-put (xwem-osd-instance-plist osin) prop val)))
+(put 'xwem-osd-instance-put-prop 'lisp-indent-function 2)
+
+(defsubst xwem-osd-instance-get-prop (osin prop)
+  "Return OSD's instance OSIN property PROP."
+  (plist-get (xwem-osd-instance-plist osin) prop))
+
+(defsubst xwem-osd-instance-rem-prop (osin prop)
+  "Remove OSD's instance OSIN property PROP."
+  (setf (xwem-osd-instance-plist osin)
+        (plist-remprop (xwem-osd-instance-plist osin) prop)))
 
 (defmacro xwem-osd-instance-xdpy (osin)
   "Return display of OSIN osd instance."
@@ -125,14 +157,10 @@
   "Return OSD's property PROP."
   (plist-get (xwem-osd-plist osd) prop))
 
-(defcustom xwem-osd-default-font "fixed"
-  "Default font for text drawed in osd.")
-
-(defcustom xwem-osd-default-color "black"
-  "Default color used to draw.")
-
-(defcustom xwem-osd-always-ontop t
-  "*Non-nil mean that OSD's winow will be always on top.")
+(defsubst xwem-osd-rem-prop (osd prop)
+  "Remove OSD's property PROP."
+  (setf (xwem-osd-plist osd)
+        (plist-remprop (xwem-osd-plist osd) prop)))
 
 
 ;;; Functions
@@ -140,7 +168,6 @@
   "On X display XDPY and window XWIN handle X Event XEV."
   (let* ((osd (xwem-osd-get-osd xwin))
          (keymap (xwem-osd-get-prop osd 'keymap)))
-
     (when (xwem-osd-p osd)
       (X-Event-CASE xev
         (:X-DestroyNotify
@@ -148,10 +175,9 @@
 
         ((:X-KeyPress :X-ButtonPress :X-ButtonRelease)
          (when (keymapp keymap)
-           (let ((xwem-override-global-map keymap))
-             (xwem-kbd-handle-keybutton xev))))
-        ))
-    ))
+           (xwem-overriding-local-map keymap
+             (xwem-dispatch-command-xevent xev)))))
+      )))
 
 (defun xwem-osd-root-event-handler (xdpy xwin xev)
   "Root window event handler for OSD."
@@ -174,12 +200,20 @@
   (XDestroyWindow (xwem-osd-instance-xdpy osin) (xwem-osd-instance-xwin osin))
   (XFreePixmap (xwem-osd-instance-xdpy osin) (xwem-osd-instance-xmask osin))
 
+  ;; If OSD instance is 'icon, it is most possible that it has special
+  ;; GC in properties, which need to be free'ed.
+  (when (eq (xwem-osd-instance-type osin) 'icon)
+    (XFreeGC (xwem-osd-instance-xdpy osin)
+             (xwem-osd-instance-get-prop osin 'icon-gc)))
+
   (X-invalidate-cl-struct osin)
   )
 
 (defun xwem-osd-add-instance (osd depth &optional color)
   "In OSD add osd instance with background COLOR.
 Return newly created osd instance."
+  (unless depth
+    (setq depth 0))
   (unless color
     (setq color xwem-osd-default-color))
 
@@ -200,8 +234,26 @@ Return newly created osd instance."
     (xwem-osd-instance-clear osin)
 
     (push osin (xwem-osd-instances osd))
-    ;; TODO: - sort instances according to depth
 
+    ;; - Sort instances according to depth
+    ;; - Install below sibling
+    (setf (xwem-osd-instances osd)
+          (sort (xwem-osd-instances osd)
+                (lambda (s1 s2)
+                  (< (xwem-osd-instance-depth s1) (xwem-osd-instance-depth s2)))))
+
+    (let ((siblings (xwem-osd-instances osd))
+          below-sibl)
+      (while siblings
+        (if (>= (xwem-osd-instance-depth (car siblings)) depth)
+            (setq siblings nil)
+          (setq below-sibl (car siblings)))
+        (setq siblings (cdr siblings)))
+
+      (when below-sibl
+        (XConfigureWindow xdpy (xwem-osd-instance-xwin osin)
+                          (make-X-Conf :sibling (xwem-osd-instance-xwin below-sibl)
+                                       :stackmode X-Below))))
     osin))
 
 (defun xwem-osd-instance-clear (osin)
@@ -231,7 +283,9 @@ Return newly created osd instance."
 ;;;###autoload
 (defun xwem-osd-create (xdpy x y width height &optional x-parent properties)
   "On X display XDPY create new xwem osd context with +X+Y/WIDTHxHEIGHT geometry on X-PARENT."
-  (let ((osd (make-xwem-osd :always-ontop xwem-osd-always-ontop :xdpy xdpy :x x :y y :width width :height height
+  (let ((osd (make-xwem-osd :always-ontop xwem-osd-always-ontop
+                            :xdpy xdpy
+                            :x x :y y :width width :height height
                             :plist properties))
         (keymap (plist-get properties 'keymap)))
     (setf (xwem-osd-xwin osd)
@@ -240,7 +294,11 @@ Return newly created osd instance."
 			 (make-X-Attr :override-redirect t
 				      :background-pixel (XBlackPixel xdpy)
 				      :event-mask (Xmask-or XM-StructureNotify
-                                                            (if keymap (Xmask-or XM-KeyPress XM-ButtonPress XM-ButtonRelease) 0)))))
+                                                            (if keymap
+                                                                (Xmask-or XM-KeyPress
+                                                                          XM-ButtonPress
+                                                                          XM-ButtonRelease)
+                                                              0)))))
     ;; Create gc
     (setf (xwem-osd-gc osd)
 	  (XCreateGC xdpy (xwem-osd-xwin osd)
@@ -266,7 +324,7 @@ X, Y, WIDTH, HEIGHT - OSD Geometry."
   (let* ((xwem-osd-always-ontop nil)    ; for sure
          (osd (xwem-osd-create xdpy 0 0 width height nil properties)))
     (xwem-osd-clear osd)
-    (XTrayInit xdpy (xwem-osd-xwin osd))
+    (xwem-XTrayInit xdpy (xwem-osd-xwin osd))
     osd))
 
 (defun xwem-osd-get-osd (xwin)
@@ -363,17 +421,19 @@ X, Y, WIDTH, HEIGHT - OSD Geometry."
   "Show OSD's window."
   (X-XShapeMask (xwem-osd-xdpy osd) (xwem-osd-xwin osd) X-XShape-Bounding X-XShapeSet 0 0 (xwem-osd-xmask osd))
   (XMapWindow (xwem-osd-xdpy osd) (xwem-osd-xwin osd))
-  (XRaiseWindow (xwem-osd-xdpy osd) (xwem-osd-xwin osd)))
+  (XRaiseWindow (xwem-osd-xdpy osd) (xwem-osd-xwin osd))
+
+  (setf (xwem-osd-state osd) 'shown))
 
 (defun xwem-osd-hide (osd)
   "Hide OSD's window."
-  (XUnmapWindow (xwem-osd-xdpy osd) (xwem-osd-xwin osd)))
+  (XUnmapWindow (xwem-osd-xdpy osd) (xwem-osd-xwin osd))
+
+  (setf (xwem-osd-state osd) 'hidden))
 
 (defun xwem-osd-destroy-instances (osd)
   "Destroy all instances in OSD."
-  (mapcar (lambda (osin)
-	    (xwem-osd-instance-destroy osin))
-	  (xwem-osd-instances osd))
+  (mapc 'xwem-osd-instance-destroy (xwem-osd-instances osd))
   (setf (xwem-osd-instances osd) nil))
 
 (defun xwem-osd-destroy (osd &optional already-destroyed)
@@ -381,8 +441,9 @@ X, Y, WIDTH, HEIGHT - OSD Geometry."
   (xwem-osd-destroy-instances osd)
 
   (X-Win-EventHandler-rem (xwem-osd-xwin osd) 'xwem-osd-event-handler)
+  (X-Win-rem-prop (xwem-osd-xwin osd) 'osd-ctx)
   
-  ;; NOTE: Can't delet 'xwem-osd-root-event-handler because some other
+  ;; NOTE: Can't delete 'xwem-osd-root-event-handler because some other
   ;;       OSD can be with always-ontop property.
 ;  (X-Win-EventHandler-rem (XDefaultRootWindow (xwem-osd-xdpy osd)) 'xwem-osd-root-event-handler)
 
@@ -426,12 +487,13 @@ X, Y, WIDTH, HEIGHT - OSD Geometry."
 		X-XShape-Bounding X-XShapeSet 0 0 (xwem-osd-xmask osd)))
 
 (defun xwem-osd-text (osd string)
-  "In OSD's context show string.
-When MASK-ONLY is non-nil, do not performe any real drawing."
+  "In OSD's context show STRING.
+If OSD has any instances, they will be destroyed."
   (let* ((xdpy (xwem-osd-xdpy osd))
 	 (yoff (- (X-Text-height xdpy (X-Gc-font (xwem-osd-mask-gc osd)) string)
 		  (X-Text-descent xdpy (X-Gc-font (xwem-osd-mask-gc osd)) string))))
 
+    (xwem-osd-destroy-instances osd)
     ;; Update window shape
     (xwem-osd-clear-mask osd)
     (XDrawString xdpy (xwem-osd-xmask osd) (xwem-osd-mask-gc osd)
@@ -444,22 +506,21 @@ When MASK-ONLY is non-nil, do not performe any real drawing."
   "In OSD's win draw colored text specified by STRSPEC-LIST."
   (xwem-osd-clear osd)
   (let ((curstr ""))
-    (X-Dpy-send-excursion (xwem-osd-xdpy osd)
-      (mapcar (lambda (strspec)
-		(let* ((xdpy (xwem-osd-xdpy osd))
-		       (str (concat curstr (car strspec)))
-		       (yoff (- (X-Text-height xdpy (X-Gc-font (xwem-osd-mask-gc osd)) str)
-				(X-Text-descent xdpy (X-Gc-font (xwem-osd-mask-gc osd)) str))))
-		  (xwem-osd-set-xwin-color osd (cdr strspec))
-		  (XDrawString xdpy (xwem-osd-xmask osd) (xwem-osd-mask-gc osd)
-			       0 yoff str)
-		  (X-XShapeMask xdpy (xwem-osd-xwin osd)
-				X-XShape-Bounding X-XShapeSet 0 0 (xwem-osd-xmask osd))
+    (mapcar (lambda (strspec)
+              (let* ((xdpy (xwem-osd-xdpy osd))
+                     (str (concat curstr (car strspec)))
+                     (yoff (- (X-Text-height xdpy (X-Gc-font (xwem-osd-mask-gc osd)) str)
+                              (X-Text-descent xdpy (X-Gc-font (xwem-osd-mask-gc osd)) str))))
+                (xwem-osd-set-xwin-color osd (cdr strspec))
+                (XDrawString xdpy (xwem-osd-xmask osd) (xwem-osd-mask-gc osd)
+                             0 yoff str)
+                (X-XShapeMask xdpy (xwem-osd-xwin osd)
+                              X-XShape-Bounding X-XShapeSet 0 0 (xwem-osd-xmask osd))
 		
-		  (setq curstr (concat curstr (car strspec)))))
-	      strspec-list))))
+                (setq curstr (concat curstr (car strspec)))))
+            strspec-list)))
     
-(defun xwem-osd-text-add (osd x y string &optional color)
+(defun xwem-osd-text-add (osd x y string &optional depth color)
   "In OSD's context at X Y coordinates add STRING colored with COLOR."
   (let* ((xdpy (xwem-osd-xdpy osd))
 	 (yoff (- (X-Text-height xdpy (X-Gc-font (xwem-osd-mask-gc osd)) string)
@@ -467,7 +528,7 @@ When MASK-ONLY is non-nil, do not performe any real drawing."
 	 osin)
 
     ;; Setup OSD instance
-    (setq osin (xwem-osd-add-instance osd 0 color))
+    (setq osin (xwem-osd-add-instance osd depth color))
     (setf (xwem-osd-instance-type osin) 'text)
     (XDrawString xdpy (xwem-osd-instance-xmask osin) (xwem-osd-mask-gc osd)
 		 x (+ y yoff) string)
@@ -480,7 +541,7 @@ When MASK-ONLY is non-nil, do not performe any real drawing."
 		 x (+ y yoff) string)
     (X-XShapeMask xdpy (xwem-osd-xwin osd)
 		  X-XShape-Bounding X-XShapeUnion 0 0 (xwem-osd-xmask osd))
-    ))
+    osin))
 
 (defun xwem-osd-set-line-width (osd new-line-width)
   "Set OSD's gc line width to NEW-LINE-WIDTH."
@@ -491,13 +552,13 @@ When MASK-ONLY is non-nil, do not performe any real drawing."
   (XChangeGC (xwem-osd-xdpy osd) (xwem-osd-mask-gc osd))
   )
 
-(defun xwem-osd-line-add (osd x0 y0 x1 y1 &optional color)
+(defun xwem-osd-line-add (osd x0 y0 x1 y1 &optional depth color)
   "In OSD's window add line."
   (let ((xdpy (xwem-osd-xdpy osd))
 	osin)
     
     ;; Create OSD line instance
-    (setq osin (xwem-osd-add-instance osd 0 color))
+    (setq osin (xwem-osd-add-instance osd depth color))
     (setf (xwem-osd-instance-type osin) 'line)
     (XDrawLine xdpy (xwem-osd-instance-xmask osin) (xwem-osd-mask-gc osd)
 	       x0 y0 x1 y1)
@@ -510,15 +571,15 @@ When MASK-ONLY is non-nil, do not performe any real drawing."
 	       x0 y0 x1 y1)
     (X-XShapeMask (xwem-osd-xdpy osd) (xwem-osd-xwin osd)
 		  X-XShape-Bounding X-XShapeUnion 0 0 (xwem-osd-xmask osd))
-    ))
+    osin))
 
-(defun xwem-osd-dots-add (osd dots type &optional color)
+(defun xwem-osd-dots-add (osd dots type &optional depth color)
   "In OSD's window add DOTS of TYPE."
   (let ((xdpy (xwem-osd-xdpy osd))
         osin)
 
     ;; Create OSD dots instancne
-    (setq osin (xwem-osd-add-instance osd 0 color))
+    (setq osin (xwem-osd-add-instance osd depth color))
     (setf (xwem-osd-instance-type osin) 'dots)
     (xwem-diag-plot-dots type (xwem-osd-instance-xmask osin) (xwem-osd-mask-gc osd)
                          0 (xwem-osd-height osd) dots)
@@ -531,15 +592,15 @@ When MASK-ONLY is non-nil, do not performe any real drawing."
                          0 (xwem-osd-height osd) dots)
     (X-XShapeMask (xwem-osd-xdpy osd) (xwem-osd-xwin osd)
 		  X-XShape-Bounding X-XShapeUnion 0 0 (xwem-osd-xmask osd))
-    ))
+    osin))
 
-(defun xwem-osd-arc-add (osd xarc &optional color)
+(defun xwem-osd-arc-add (osd xarc &optional depth color)
   "In OSD's window draw arc specified by XARC."
   (let ((xdpy (xwem-osd-xdpy osd))
 	osin)
 
     ;; Create OSD arc instance
-    (setq osin (xwem-osd-add-instance osd 0 color))
+    (setq osin (xwem-osd-add-instance osd depth color))
     (setf (xwem-osd-instance-type osin) 'arc)
     (XDrawArcs xdpy (xwem-osd-instance-xmask osin) (xwem-osd-mask-gc osd)
 	       (list xarc))
@@ -551,27 +612,89 @@ When MASK-ONLY is non-nil, do not performe any real drawing."
 	       (list xarc))
     (X-XShapeMask (xwem-osd-xdpy osd) (xwem-osd-xwin osd)
 		  X-XShape-Bounding X-XShapeUnion 0 0 (xwem-osd-xmask osd))
-    ))
+    osin))
 
-(defun xwem-osd-rect-add (osd x y width height &optional color)
-  "In OSD's window add rectangle specified by X Y WIDTH and HEIGHT."
+(defun xwem-osd-rect-add (osd x y width height &optional depth color fill-p)
+  "In OSD's window add rectangle specified by X Y WIDTH and HEIGHT.
+If FILL-P is non-nil, rectangle will be filled instead of outdrawing."
   (let ((xdpy (xwem-osd-xdpy osd))
 	osin)
 
     ;; Created OSD rect instance
-    (setq osin (xwem-osd-add-instance osd 0 color))
+    (setq osin (xwem-osd-add-instance osd depth color))
     (setf (xwem-osd-instance-type osin) 'rect)
-    (XDrawRectangle xdpy (xwem-osd-instance-xmask osin) (xwem-osd-mask-gc osd)
-		    x y width height)
+    (XDrawRectangles xdpy (xwem-osd-instance-xmask osin) (xwem-osd-mask-gc osd)
+                     (list (make-X-Rect :x x :y y :width width :height height))
+                     fill-p)
     (X-XShapeMask xdpy (xwem-osd-instance-xwin osin)
 		  X-XShape-Bounding X-XShapeSet 0 0 (xwem-osd-instance-xmask osin))
 
     ;; Update OSD shape
-    (XDrawRectangle xdpy (xwem-osd-xmask osd) (xwem-osd-mask-gc osd)
-		    x y width height)
+    (XDrawRectangles xdpy (xwem-osd-xmask osd) (xwem-osd-mask-gc osd)
+                     (list (make-X-Rect :x x :y y :width width :height height))
+                     fill-p)
     (X-XShapeMask (xwem-osd-xdpy osd) (xwem-osd-xwin osd)
 		  X-XShape-Bounding X-XShapeUnion 0 0 (xwem-osd-xmask osd))
-    ))
+    osin))
+
+(defun xwem-osd-icon-data-add (osd xpm-data &optional x y depth)
+  "In OSD's window add icon.
+X and Y specifies osd instance location inside OSD(default is 0 0).
+DEPTH specifies osd instance depth(default is 0).
+XPM-DATA string of xpm image."
+  (unless depth
+    (setq depth 0))
+  (unless x
+    (setq x 0))
+  (unless y
+    (setq y 0))
+
+  (let ((xdpy (xwem-osd-xdpy osd))
+	ximg ximg-shape osin gc)
+
+    (setq ximg (X:xpm-img-from-data xdpy xpm-data)
+          ximg-shape (X:xpm-img-from-data xdpy xpm-data t))
+
+    ;; Created OSD icon instance
+    (setq osin (xwem-osd-add-instance osd depth))
+    (setf (xwem-osd-instance-type osin) 'icon)
+
+    (setq gc (XCreateGC xdpy (xwem-osd-instance-xmask osin)
+			(make-X-Gc :dpy xdpy :id (X-Dpy-get-id xdpy)
+				   :foreground 1.0
+				   :background 0.0)))
+    (when ximg-shape
+      (XImagePut xdpy gc (xwem-osd-instance-xmask osin) x y ximg-shape)
+      (X-XShapeMask xdpy (xwem-osd-instance-xwin osin)
+                    X-XShape-Bounding X-XShapeSet 0 0 (xwem-osd-instance-xmask osin))
+
+      ;; Update OSD shape
+      (XImagePut xdpy gc (xwem-osd-xmask osd) x y ximg-shape)
+      (X-XShapeMask (xwem-osd-xdpy osd) (xwem-osd-xwin osd)
+                    X-XShape-Bounding X-XShapeUnion 0 0 (xwem-osd-xmask osd)))
+    (XFreeGC xdpy gc)
+
+    ;; Draw Image
+    (setq gc (XCreateGC xdpy (xwem-osd-instance-xwin osin)
+			(make-X-Gc :dpy xdpy :id (X-Dpy-get-id xdpy)
+				   :foreground 1.0
+				   :background 0.0)))
+    (xwem-osd-instance-put-prop osin 'icon-gc gc)
+    (xwem-osd-instance-put-prop osin 'ximg ximg)
+    (xwem-osd-instance-put-prop osin 'ximg-shape ximg-shape)
+
+    (xwem-osd-instance-show osin)
+    (XImagePut xdpy gc (xwem-osd-instance-xwin osin) x y ximg)
+    osin))
+
+(defun xwem-osd-icon-file-add (osd xpm-file &optional x y depth)
+  "Same as `xwem-osd-icon-data-add', but takes xpm image from FILE."
+  (let (xpm-data)
+    (with-temp-buffer
+      (insert-file-contents-literally xpm-file)
+      (setq xpm-data (buffer-substring)))
+
+  (xwem-osd-icon-data-add osd xpm-data x y depth)))
 
 ;;; You might consider more powerfull `working' package, which is part
 ;;; of CEDET.
@@ -617,42 +740,77 @@ TOT-LEN is the total length of bar.  PERCENTS is percentage state."
     
 ;;  (xwem-osd-destroy mosd)
 
-;; Here is example from Steve Youngs <sryoungs@bigpond.net.au> to
+;; Here is example from Steve Youngs <steve@xwem.org> to
 ;; display date using OSD:
 
-;;   (defvar sy-osd-date ni)
-;;   (copy-face 'default 'sy-osd-date-face)
-;;   (set-face-foreground 'sy-osd-date-face "cyan")
-;; 
-;;   (defun sy-update-osd-date-maybe (&optional force-update)
-;;     (let* ((now (decode-time))
-;;            (cur-hour (nth 2 now))
-;;            (cur-min (nth 1 now))
-;;            (cur-comp-time (+ (* cur-hour 60) cur-min)))
-;;       (when (or force-update (= 0 cur-comp-time))
-;;         (xwem-osd-text sy-osd (format-time-string "%a, %b%e")))))
-;;     
-;;   (defun sy-show-date-osd ()
-;;     (let* ((fromleft 830)
-;;            (fromtop 740)
-;;            (face `sy-osd-face)
-;;            (xwem-osd-always-ontop t))     ; Force date to be always ontop
-;;       (setq sy-osd (xwem-osd-create (xwem-dpy) fromleft fromtop 400 200))
-;;       (xwem-osd-set-color sy-osd (face-foreground-name face))
-;;       (xwem-osd-set-font sy-osd (face-font-name face))
-;;       (sy-update-osd-date-maybe t)
-;;       (xwem-osd-show sy-osd)
-;;   
-;;       (start-itimer "sy-osd-date-itimer"
-;;                     'sy-update-osd-date-maybe
-;;                     60 60)))
-;;   
-;;   (defun sy-destroy-date-osd ()
-;;     (delete-itimer  "sy-osd-date-itimer")
-;;     (when (xwem-osd-p sy-osd)
-;;       (xwem-osd-destroy sy-osd)))
-;;     
-;;   (add-hook 'xwem-after-init-hook 'sy-show-date-osd)
+;; (require 'xwem-osd)
+;; (defvar sy-osd-date nil)
+;; (copy-face 'default 'sy-osd-date-face)
+;; (set-face-foreground 'sy-osd-date-face "blanchedalmond")
+
+;; (defvar sy-osd-date-keymap
+;;   (let ((map (make-sparse-keymap 'sy-osd-date-keymap)))
+;;     (define-key map [button3] '(lambda () (interactive) (calendar)))
+;;     map)
+;;   "Keymap for date OSD.")
+
+;; (defun sy-show-date-osd ()
+;;   "*Display the current date using OSD."
+;;   (interactive)
+;;   (let* ((face `sy-osd-date-face)
+;; 	 (text (format-time-string "%a, %b %e"))
+;; 	 (width (+ 3 (X-Text-width
+;; 		      (xwem-dpy)
+;; 		      (X-Font-get (xwem-dpy)
+;; 				  (face-font-name face))
+;; 		      text)))
+;; 	 (height (+ 3 (X-Text-height
+;; 		       (xwem-dpy)
+;; 		       (X-Font-get (xwem-dpy)
+;; 				   (face-font-name face))
+;; 		       text))))
+;;     (setq sy-osd-date (xwem-osd-create-dock
+;; 		       (xwem-dpy)
+;; 		       width
+;; 		       height
+;; 		       (list 'keymap sy-osd-date-keymap)))
+;;     (xwem-osd-set-color sy-osd-date (face-foreground-name face))
+;;     (xwem-osd-set-font sy-osd-date (face-font-name face))
+;;     (xwem-osd-text sy-osd-date text)
+;;     (xwem-osd-show sy-osd-date)))
+
+;; (defun sy-update-osd-date-maybe (&optional force)
+;;   "Update the OSD date at midnight.
+
+;;  Optional Argument FORCE means to update the date now."
+;;   (let* ((now (decode-time))
+;; 	 (cur-hour (nth 2 now))
+;; 	 (cur-min (nth 1 now))
+;; 	 (cur-comp-time (+ (* cur-hour 60) cur-min)))
+;;     (when (or force (= 0 cur-comp-time))
+;;       (when (xwem-osd-p sy-osd-date)
+;; 	(xwem-osd-text sy-osd-date (format-time-string "%a, %b %e"))))))
+
+;; (defun sy-update-osd-date ()
+;;   "*Force update of the OSD date."
+;;   (interactive)
+;;   (when (xwem-osd-p sy-osd-date)
+;;     (sy-update-osd-date-maybe t)))
+
+;; (defun sy-delete-osd-date ()
+;;   "*Delete the OSD date."
+;;   (interactive)
+;;   (when (xwem-osd-p sy-osd-date)
+;;     (when (itimerp "sy-osd-date-itimer")
+;;       (delete-itimer "sy-osd-date-itimer"))
+;;     (xwem-osd-destroy sy-osd-date)))
+
+;; (add-hook 'xwem-after-init-hook (lambda ()
+;; 				  (progn
+;; 				    (sy-show-date-osd)
+;; 				    (start-itimer "sy-osd-date-itimer"
+;; 						  'sy-update-osd-date-maybe
+;; 						  60 60))))
 
 
 
